#!/bin/sh

# LOG_LEVEL
#   0  Only fatal error messages
#   1  Description of the current process
#   2  Solvers iterations are shown (default)
export LOG_LEVEL=2

# SOLVER_TYPE
# Chooses solver for all session
#   1  Conjugate gradient (default)
#   2  Cholesky decomposition
#   3  Cholesky2 decomposition
#   4  Biconjugate gradient
#   5  LU decomposition
export SOLVER_TYPE=1

# SOLVER_THREADS
# Sets the maximum number of threads used by solvers
export SOLVER_THREADS=1

# SOLVER_TOLERANCE
# For iterative solvers, this defines the convergence criteria
export SOLVER_TOLERANCE=1e-5

# SOLVER_MAX_STEPS
# For iterative solvers, it is the maximum number of iterations allowed
export SOLVER_MAX_STEPS=10000

# PRECONDITIONER_TYPE
# For iterative solvers.
#   0=None
#   1=Jacobi (default)
#   2=Incomplete_Cholesky (Only for Conjugate gradient)
#   3=Incomplete_Cholesky2 (Only for Conjugate gradient)
#   4=Incomplete_LU (Only for Biconjugate gradient)
#   5=Sparse_Approximate_Inverse
export PRECONDITIONER_TYPE=1

# PRECONDITIONER_LEVEL
# Set the preconditioner level for Incomplete_Cholesky, Incomplete_Cholesky2, Incomplete_LU and Sparse_Approximate_Inverse
export PRECONDITIONER_LEVEL=1

# PRECONDITIONER_THRESHOLD
# Set the preconditioner threshold for Sparse_Approximate_Inverse
export PRECONDITIONER_THRESHOLD=0.0

# DOMAIN_DECOMPOSITION_METHOD
# Choose the domain decomposition method to use
#   1=Schwarz_alternating
#   2=Reaction_alternating
export DOMAIN_DECOMPOSITION_METHOD=1

# DECOMPOSITION_THREADS
# Set the number of threads used to assemble data for slave processes
export DECOMPOSITION_THREADS=1

# GLOBAL_TOLERANCE
# Tolerance for global convergence of the domain decomposition method
export GLOBAL_TOLERANCE=1e-5

# UPDATE_FACTOR
# Defines a weight for update boundary values
export UPDATE_FACTOR=0.9

# SCHWARZ_ALTERNATING_OVERLAPPING
# Defines the number of layers of overlapping among partitions
export SCHWARZ_ALTERNATING_OVERLAPPING=2

named_pipe_data="/tmp/FEMData"
named_pipe_result="/tmp/FEMResult"

# PROCESSES
export processes=3

mpiexec -x LOG_LEVEL -x SOLVER_TYPE -x SOLVER_THREADS -x SOLVER_TOLERANCE -x SOLVER_MAX_STEPS -x PRECONDITIONER_TYPE -x PRECONDITIONER_LEVEL -x PRECONDITIONER_THRESHOLD -x DOMAIN_DECOMPOSITION_METHOD -x DECOMPOSITION_THREADS -x GLOBAL_TOLERANCE -x UPDATE_FACTOR -x SCHWARZ_ALTERNATING_OVERLAPPING -n $processes FEMSolver.MPI $named_pipe_data $named_pipe_result
